-----------------------------------------------------------------------------------------------
-- Client Lua Script for SellSalvageKeep
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "GameLib"
require "Item"
require "Window"
require "Sound"

 
-----------------------------------------------------------------------------------------------
-- SellSalvageKeep Module Definition
-----------------------------------------------------------------------------------------------
local SellSalvageKeep = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local maxLoot = 4
local knType_Invalid = 0
local knType_Item = 1
local kfMaxItemTime = 20	-- item display time (seconds)
local kfTimeBetweenItems = 2 -- Previously .3			-- delay between items; also determines clearing time (seconds)


local karQualitySquareSprite =
{
	[Item.CodeEnumItemQuality.Inferior] 		= "CRB_Tooltips:sprTooltip_Header_Silver",
	[Item.CodeEnumItemQuality.Average] 			= "CRB_Tooltips:sprTooltip_Header_White",
	[Item.CodeEnumItemQuality.Good] 			= "CRB_Tooltips:sprTooltip_Header_Green",
	[Item.CodeEnumItemQuality.Excellent] 		= "CRB_Tooltips:sprTooltip_Header_Blue",
	[Item.CodeEnumItemQuality.Superb] 			= "CRB_Tooltips:sprTooltip_Header_Purple",
	[Item.CodeEnumItemQuality.Legendary] 		= "CRB_Tooltips:sprTooltip_Header_Orange",
	[Item.CodeEnumItemQuality.Artifact]		 	= "CRB_Tooltips:sprTooltip_Header_Pink",
}

local karEvalColors =
{
	[Item.CodeEnumItemQuality.Inferior] 		= "ItemQuality_Inferior",
	[Item.CodeEnumItemQuality.Average] 			= "ItemQuality_Average",
	[Item.CodeEnumItemQuality.Good] 			= "ItemQuality_Good",
	[Item.CodeEnumItemQuality.Excellent] 		= "ItemQuality_Excellent",
	[Item.CodeEnumItemQuality.Superb] 			= "ItemQuality_Superb",
	[Item.CodeEnumItemQuality.Legendary] 		= "ItemQuality_Legendary",
	[Item.CodeEnumItemQuality.Artifact]		 	= "ItemQuality_Artifact",
}

 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
local knSaveVersion = 0

function SellSalvageKeep:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.arEntries = {}
	o.tSaveData = {}
	o.tEntryData = {}
	o.tQueuedEntryData = {}
	o.fLastTimeAdded = 0
	o.dongs = ""
	o.currentCash = 0
	o.tItemSell = {}
	o.tItemSalv = {}
	o.tItemSellIDs = {}
	o.tItemSalvIDs = {}
	o.nSalvIndex = nil
	-- options menu
	-- vac menu
	o.craftRadio = true
	o.equipRadio = true
	o.decorRadio = true
	o.runeRadio = true
	o.consumeRadio = true
	o.ampRadio = true
	o.bagRadio = true
	o.costumeRadio = true
	o.pathRadio = true
	o.miscRadio = true
	o.schematicRadio = true
	o.warplotRadio = true
	-- coin purse
	o.shownRadio = false
	o.noShowRadio = false
    return o
end

function SellSalvageKeep:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SellSalvageKeep OnLoad
-----------------------------------------------------------------------------------------------
function SellSalvageKeep:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SellSalvageKeep.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SellSalvageKeep OnDocLoaded
-----------------------------------------------------------------------------------------------
function SellSalvageKeep:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "TutBase", nil, self)
		self.wndLoot = Apollo.LoadForm(self.xmlDoc, "Looter", nil, self)
		self.wndSelect = Apollo.LoadForm(self.xmlDoc, "ViewSelect", nil, self)
		self.wndSellList = Apollo.LoadForm(self.xmlDoc, "ItemList", nil, self)
		self.wndSalvList = Apollo.LoadForm(self.xmlDoc, "ItemList", nil, self)
		self.wndOption = Apollo.LoadForm(self.xmlDoc, "Options", nil, self)
		--self.wndOptionFill = Apollo.LoadForm(self.xmlDoc, "OptionsFill", self.wndOption, self)
		self.wndPurse = Apollo.LoadForm(self.xmlDoc, "CoinPurse", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.wndMain:Show(false, true)
		self.wndOption:Show(false, true)
		self.wndSelect:Show(false,true)
		self.wndLoot:Show(false,true)
		self.wndSellList:Show(false,true)
		self.wndSalvList:Show(false, true)
		self.wndPurse:Show(false, true)
		self.wndOption:FindChild("Window"):Show(true)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterEventHandler("PlayerCurrencyChanged", "OnMoneyChange", self)
		Apollo.RegisterEventHandler("LootedMoney", "OnLootedMoney", self)
		Apollo.RegisterTimerHandler("SellSalvageKeep_CashTimer", 	"OnSellSalvageKeep_CashTimer", self)
		Apollo.CreateTimer("SellSalvageKeep_CashTimer", 5.0, false)
		Apollo.StartTimer("SellSalvageKeep_CashTimer")
			
		Apollo.RegisterSlashCommand("sskVac", "OnSellSalvageKeepVacOn", self)
		Apollo.RegisterSlashCommand("ssk", "OnSellSalvageKeepOn", self)
		Apollo.RegisterEventHandler("LootedItem", "OnItemGather", self)
		Apollo.RegisterEventHandler("InvokeVendorWindow", "OnVendorOpen", self)
		Apollo.RegisterEventHandler("sskInterfaceMenuClicked", "OnSellSalvageKeepOn", self)
		Apollo.RegisterTimerHandler("LootUpdate",		"OnLootUpdate", self)
		Apollo.CreateTimer("LootUpdate", 0.1, true)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "RestoreDataLists", self)
		
		self.wndPurseTempWindow = self.wndPurse:FindChild("PurseTempMoney:PurseMoneyWindow")
		self.wndPurseWindow = self.wndPurse:FindChild("PurseCashMoney:PurseMoneyWindow")
		self.wndMoneyWindow = self.wndLoot:FindChild("LootFloating:CashMoney:MoneyWindow")
		self.wndCashMoney = self.wndLoot:FindChild("LootFloating:CashMoney")
		self.wndCashMoney:Show(false)
		self.wndPurse:FindChild("mino"):Show(false)
		self.wndPurse:FindChild("mino2"):Show(false)
		

		for idx = 1, maxLoot do
			local wndEntry = self.wndLoot:FindChild("LootFloating:LootItem_"..idx)
			wndEntry:Show(false, true)
			self.arEntries[idx] = wndEntry
		end
		local wndEntry = self.wndLoot:FindChild("LootFloating:LootItem_4")
		wndEntry:Show(false, true)
		self:UpdateDisplay()
		
		self.SalvListItemRemover = nil
		--options init
		
		--coin purse
		self.wndOption:FindChild("OptionsFill:coinBase:shownNo"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:coinBase:noShowNo"):SetCheck(true)
		--vac
		self.wndOption:FindChild("OptionsFill:vacWnd1base:craftYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:equipYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:decorYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:RuneYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:ConsumeYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:AmpYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:BagYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:CostumeYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:PathYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:MiscYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:SchematicYes"):SetCheck(true)
		self.wndOption:FindChild("OptionsFill:vacWnd1base:WarYes"):SetCheck(true)
		-- options menu
		-- vac menu
		--self.craftRadio = true
		--self.equipRadio = true
		--self.decorRadio = true
		--self.runeRadio = true
		--self.consumeRadio = true
		-- coin purse
		--self.shownRadio = false

	end
end

-----------------------------------------------------------------------------------------------
-- SellSalvageKeep Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/ssk"
function SellSalvageKeep:OnSellSalvageKeepOn()
	self.wndMain:Invoke() -- show the window
	--Print(self.dongs)
end

function SellSalvageKeep:OnSellSalvageKeepVacOn()
	if self.wndSelect:IsShown() == true then
		self.wndSelect:Show(false, true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	end
	self.wndLoot:Invoke()
	--Self:UpdateSalv()
end
-----------------------------------------------------------------------------------------------
-- CASH FUNCTIONS
-----------------------------------------------------------------------------------------------

function SellSalvageKeep:OnLootedMoney(monLooted)
	local eCurrencyType = monLooted:GetMoneyType()
	if eCurrencyType ~= Money.CodeEnumCurrencyType.Credits then
		return
	end
	if self.wndLoot:IsShown() and self.shownRadio == false then
		self.wndMoneyWindow:SetAmount(self.wndMoneyWindow:GetAmount() + monLooted:GetAmount())
		self.wndCashMoney:Invoke()
	elseif self.noShowRadio == true then
		--do nothing
	else
		self.wndPurse:Invoke()
	end
	--self.wndPurseTempWindow:SetAmount(self.wndPurseTempWindow:GetAmount() + monLooted:GetAmount())
	
	
	if self.wndPurse:FindChild("mino"):IsShown() then
		local tempTotal = self.wndPurseTempWindow:GetAmount()
		if tempTotal > monLooted:GetAmount() then
			self.wndPurseTempWindow:SetAmount(tempTotal - monLooted:GetAmount())
		else
			self.wndPurseTempWindow:SetAmount(monLooted:GetAmount() - tempTotal)
			self.wndPurse:FindChild("mino"):Show(false)
		end
	else
		self.wndPurseTempWindow:SetAmount(self.wndPurseTempWindow:GetAmount() + monLooted:GetAmount())
	end
	
	
	if self.wndPurse:FindChild("mino2"):IsShown() then
		local sessionTotal = self.wndPurseWindow:GetAmount()
		if sessionTotal > monLooted:GetAmount() then
			self.wndPurseWindow:SetAmount(sessionTotal - monLooted:GetAmount())
		else
			self.wndPurseWindow:SetAmount(monLooted:GetAmount() - sessionTotal)
			self.wndPurse:FindChild("mino2"):Show(false)
		end
	else
		self.wndPurseWindow:SetAmount(self.wndPurseWindow:GetAmount() + monLooted:GetAmount())
	end
	
	
	
	Apollo.StopTimer("SellSalvageKeep_CashTimer")
	Apollo.StartTimer("SellSalvageKeep_CashTimer")
	self.currentCash = GameLib.GetPlayerCurrency():GetAmount()
end
function SellSalvageKeep:OnSellSalvageKeep_CashTimer()
	self.wndCashMoney:Show(false)
	self.wndMoneyWindow:SetAmount(0)
	self.wndPurseTempWindow:SetAmount(0)
	self.wndPurse:FindChild("mino"):Show(false)
end
-------------------------------------------------------------------------------------------------
-- on LootedItem
-------------------------------------------------------------------------------------------------
function SellSalvageKeep:OnItemGather(itemInstance, nCount)
	local tNewEntry =
	{
		eType = knType_Item,
		itemInstance = itemInstance,
		nCount = nCount,
		money = nil,
		fTimeAdded = GameLib.GetGameTime()
		--tt
	}
	--Print(itemInstance:GetItemFamilyName())
	--Print(itemInstance:GetItemCategoryName())
	--Print(itemInstance:GetItemTypeName())	
	--puts a new entry into the queue
	local showAndAddFlag = false
	if itemInstance:GetItemCategoryName() ~= "Junk" then	
		if self.wndPurse:IsShown()== true and self.shownRadio == false then
			self.wndPurse:Show(false, true)
		end
		if self.craftRadio == true and itemInstance:GetItemFamilyName() == "Crafting" and itemInstance:GetItemCategoryName() ~= "Elements" and itemInstance:GetItemTypeName() ~= "Rune Fragment" then
			--show vac and add item to list 
			showAndAddFlag = true
		end
		if self.equipRadio == true and (itemInstance:GetItemFamilyName() == "Gear" or itemInstance:GetItemFamilyName() == "Armor" or itemInstance:GetItemFamilyName() == "Weapon") then
			-- show vac and add item to list
			showAndAddFlag = true
		end
		if self.decorRadio == true and itemInstance:GetItemTypeName() == "Decor" then
			-- show vac and add item to list
			showAndAddFlag = true
		end
		if self.runeRadio == true and (itemInstance:GetItemCategoryName() == "Elements" or itemInstance:GetItemTypeName() == "Rune Fragment") then
			--show vac and add item to list
			showAndAddFlag = true
		end
		if self.consumeRadio == true and itemInstance:GetItemFamilyName() == "Consumable" then
			-- show vac and add item to list
			showAndAddFlag = true
		end
		if self.ampRadio == true and itemInstance:GetItemFamilyName() == "AMP" then
			showAndAddFlag = true
		end
		if self.bagRadio == true and itemInstance:GetItemFamilyName() == "Bag" then
			showAndAddFlag = true
		end
		if self.costumeRadio == true and itemInstance:GetItemFamilyName() == "Costume" then
			showAndAddFlag = true
		end
		if self.pathRadio == true and itemInstance:GetItemFamilyName() == "Path" then
			showAndAddFlag = true
		end
		if self.miscRadio == true and (itemInstance:GetItemFamilyName() == "Miscellaneous" or itemInstance:GetItemFamilyName() == "Unusual Component") then
			showAndAddFlag = true
		end
		if self.schematicRadio == true and itemInstance:GetItemFamilyName() == "Schematic" then
			showAndAddFlag = true
		end
		if self.warplotRadio == true and itemInstance:GetItemFamilyName() == "Warplot" then
			showAndAddFlag = true
		end
		
		if showAndAddFlag == true then
			self.wndLoot:Invoke()
			table.insert(self.tQueuedEntryData, tNewEntry)
			self.fLastTimeAdded = GameLib.GetGameTime()
			showAndAddFlag = false
		end
	else	
		table.insert(self.tItemSell, itemInstance)
	end
end

function SellSalvageKeep:OnLootUpdate(strVar, nValue)
	if self.wndLoot == nil then
		return
	end
	
	local fCurrTime = GameLib.GetGameTime()

	-- remove any old items
	--for idx, tEntryData in ipairs(self.tEntryData) do   --TODO: time the remove to delay
		--if fCurrTime - tEntryData.fTimeAdded >= kfMaxItemTime then
			--self:RemoveItem(idx)
		--end
	--end

	-- add a new item if its time
	if #self.tQueuedEntryData > 0 then
		--if fCurrTime - self.fLastTimeAdded >= kfTimeBetweenItems then
		self:AddQueuedItem()
		--end
	end

	-- update all the items
	self:UpdateDisplay()
end

function SellSalvageKeep:AddQueuedItem()
	-- gather our entryData we need
	--get the first item in the table queue
	local tQueuedData = self.tQueuedEntryData[1]
	--remove the item from the table queue
	table.remove(self.tQueuedEntryData, 1)
	--make sure something was found
	if tQueuedData == nil then
		return
	end

	if tQueuedData.eType == knType_Item and tQueuedData.nCount == 0 then
		return
	end

	-- ensure there's room
	--while #self.tEntryData >= 4 do
		--if not self:RemoveItem(1) then
			--break
		--end
	--end

	-- push this item on the end of the table
	local fCurrTime = GameLib.GetGameTime()
	local nBtnIdx = #self.tEntryData + 1
	self.tEntryData[nBtnIdx] = tQueuedData
	self.tEntryData[nBtnIdx].fTimeAdded = fCurrTime -- adds a delay for vaccuum looting by switching logged to "shown" time

	self.fLastTimeAdded = fCurrTime

	-- animate the entry down
	--self.arEntries[btnIdx]:PlayAnim(0)
end

function SellSalvageKeep:RemoveItem(idx)
	-- validate our inputs
	if idx < 1 or idx > #self.tEntryData then
		return false
	end

	-- remove that item and alert inventory
	local tEntryData = self.tEntryData[idx]
	table.remove(self.tEntryData, idx)
	return true
end

function SellSalvageKeep:UpdateDisplay()
	-- iterate over our entry data updating all the buttons
	for idx, wndEntry in ipairs(self.arEntries) do
		local tCurrEntryData = self.tEntryData[idx]
		local tCurrItem = tCurrEntryData and tCurrEntryData.itemInstance or false

		if tCurrEntryData then
			wndEntry:Invoke()
		else
			wndEntry:Close()
		end
		if tCurrEntryData and tCurrItem and tCurrEntryData.nButton ~= idx then
			local bGivenQuest = tCurrItem:GetGivenQuest()
			local eItemQuality = tCurrItem and tCurrItem:GetItemQuality() or 1
			local cItemTooltipForm = Tooltip.GetItemTooltipForm(self, wndEntry:FindChild("LootSack"),tCurrItem, {bPrimary = true, bSelling = false, itemCompare = itemEquipped})
			wndEntry:FindChild("LootText"):SetTextColor(bGivenQuest and "white" or karEvalColors[eItemQuality])
			wndEntry:FindChild("LootSack"):SetSprite(bGivenQuest and "sprMM_QuestGiver" or tCurrItem:GetIcon())
			wndEntry:FindChild("LootSack"):SetTooltipForm(cItemTooltipForm)
			wndEntry:FindChild("RarityBracket"):SetSprite(bGivenQuest and "sprTooltip_Header_White" or karQualitySquareSprite[eItemQuality])
			
			local itemPrice = tCurrItem:GetSellPrice()
			--if tCurrItem:CanSalvage
			--Print(wndEntry:FindChild("MiniSalv_"..idx):GetName())
			--Print(itemPrice)
			if tCurrItem:CanSalvage() == false then	
				wndEntry:FindChild("MiniSalv_"..idx):Show(false,true)
			else
				wndEntry:FindChild("MiniSalv_"..idx):Show(true,true)
			end
			--Print(tCurrItem:GetItemFamilyName())
			--Print(tCurrItem:GetItemCategoryName())
			--Print(tCurrItem:GetItemTypeName())
			if itemPrice == nil then
				wndEntry:FindChild("MiniSell_"..idx):Show(false,true)
			else
				wndEntry:FindChild("MiniSell_"..idx):Show(true,true)
			end
			if tCurrItem:GetItemFamilyName() == "Crafting" and tCurrItem:GetItemCategoryName() ~= "Crafting Supplies"then
				wndEntry:FindChild("MiniSell_"..idx):Show(false,true)
				wndEntry:FindChild("MiniSalv_"..idx):Show(false,true)
				--wndEntry:FindChild("MiniKeep_"..idx):Show(false,true)
			end
			--Print(tCurrItem:GetName())
			--sets number of items picked up ex. 3-omniplasm
			if tCurrEntryData.nCount == 1 then
				wndEntry:FindChild("LootText"):SetText(tCurrItem:GetName())
			else
				wndEntry:FindChild("LootText"):SetText(String_GetWeaselString(Apollo.GetString("CombatLog_MultiItem"), tCurrEntryData.nCount, tCurrItem:GetName()))
			end
			tCurrEntryData.nButton = idx
		end
	end

	self.wndLoot:FindChild("LootFloating"):ArrangeChildrenVert(2)
end
-----------------------------------------------------------------------------------------------
--vendoring
-----------------------------------------------------------------------------------------------
function SellSalvageKeep:OnVendorOpen(uVendorUnitId)
	local vendorUnit = uVendorUnitId
	local vendorUnitName = vendorUnit:GetName()		
	--Print(vendorUnitName)
	self.currentCash = GameLib.GetPlayerCurrency():GetAmount()
	--Print(self.currentCash)
	if #self.tItemSell > 0 then
		self.wndSellList:DestroyChildren()
		--for each item in table sell to vendor
		for key,value in pairs(self.tItemSell) do 
			local tempItem = value
			--Print(tempItem:GetName())
			--Print(tempItem:GetStackCount())
			SellItemToVendorById(tempItem:GetInventoryId(), tempItem:GetStackCount())
			self.tItemSell[key] = nil
		end
	end
	
end

function SellSalvageKeep:OnMoneyChange()
	local newMoney = GameLib.GetPlayerCurrency():GetAmount()

	if newMoney < self.currentCash then
		local diff = self.currentCash - newMoney	
		if self.wndPurse:IsShown() == false then	
			self.wndPurse:Invoke()
		end
		
		
		local tempMoney = self.wndPurseTempWindow:GetAmount()
		local tempDiff = tempMoney - diff
		if self.wndPurse:FindChild("mino"):IsShown() then
			self.wndPurseTempWindow:SetAmount(diff + self.wndPurseTempWindow:GetAmount())
		else
			if tempMoney >= diff then
				self.wndPurseTempWindow:SetAmount(tempDiff)
			else
				self.wndPurseTempWindow:SetAmount(diff - self.wndPurseTempWindow:GetAmount())
				self.wndPurse:FindChild("mino"):Show(true)
			end
		end
		
		
		
		local sessionMoney = self.wndPurseWindow:GetAmount()
		local sessionDiff = sessionMoney - diff
		if self.wndPurse:FindChild("mino2"):IsShown() then
			self.wndPurseWindow:SetAmount(diff + self.wndPurseWindow:GetAmount())			
		else
			if sessionMoney >= diff then
				self.wndPurseWindow:SetAmount(sessionDiff)
			else			
				self.wndPurseWindow:SetAmount(diff - self.wndPurseWindow:GetAmount())
				self.wndPurse:FindChild("mino2"):Show(true)
			end
		end


		Apollo.StopTimer("SellSalvageKeep_CashTimer")
		Apollo.StartTimer("SellSalvageKeep_CashTimer")

	end
	self.currentCash = GameLib.GetPlayerCurrency():GetAmount()
end

-----------------------------------------------------------------------------------------------
-- LooterForm Functions
-----------------------------------------------------------------------------------------------
function SellSalvageKeep:OnClose()
	if self.wndSelect:IsShown()== true then
		self.wndSelect:Show(false,true)
	end	
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	end	
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndOption:IsShown() == true then
		self.wndOption:Show(false, true)
	end
	self.wndLoot:Close()
end

function SellSalvageKeep:OnSellALL()
	--loop through each window
		--get item
		--if an item exists and has a sell price
			--add item to sell list
			--remove item from window
			--update display
	--currently sells window item 1,3 not 2,4
	local inventoryItemTable = GameLib.GetPlayerUnit():GetInventoryItems()
	local tIndex = {0,0,0,0}
	for idx = 1, 4 do		
		local tCurrEntryData = self.tEntryData[idx]
		local tCurrItem = tCurrEntryData and tCurrEntryData.itemInstance or false
		local itemPrice
		if tCurrItem ~= false then
			itemPrice = tCurrItem:GetSellPrice()
		end	
		--local inventoryItemTable = GameLib.GetPlayerUnit():GetInventoryItems()	
		if tCurrItem and tCurrItem:GetItemFamilyName() ~= "Crafting" and itemPrice ~= nil then
			--Print("item added to list "..idx)
			for key, item in pairs (inventoryItemTable) do	
				local tempItem = item
				if tempItem and tempItem.itemInBag and tempItem.itemInBag:GetName() == tCurrItem:GetName() then
					table.insert(self.tItemSell, tCurrItem)
					--Print(tIndex[idx])	
					break
				end
			end
			--self:RemoveItem(idx)
			tIndex[idx] = idx
				
			--self:UpdateDisplay()
		else
			--Print("item not added to list "..idx)
		end
	end
	--could be a seperate method
	for idxTwo = 4,1,-1 do	
		if tIndex[idxTwo] ~= 0 then
			self:RemoveItem(idxTwo)
			self:UpdateDisplay()
		end
	end
	self:NoItemClose()
end

function SellSalvageKeep:OnSalvAll()
	
	if self.nSalvIndex == nil then
		self.nSalvIndex = 1
	end
	--loop through each window
		--set nSalvIndex to 1
		--get item
		--if an item exists and can be salvaged
			--add item to salv list
			--remove item from window
			--update display
			--updatesalv
	local tIndex = {0,0,0,0}
	self.nSalvIndex = 1
	--Print(self.nSalvIndex)
	local inventoryItemTable = GameLib.GetPlayerUnit():GetInventoryItems()
	for idx = 1, 4 do	
		--nSalvIndex = 1	
		local tCurrEntryData = self.tEntryData[idx]
		local tCurrItem = tCurrEntryData and tCurrEntryData.itemInstance or false
	
		if tCurrItem and tCurrItem:CanSalvage() == true then
			--Print("item kept "..idx)
			--table.insert(self.tItemSalv, tCurrItem)
			tIndex[idx] = idx		
			local salvUpFlag = false
			for key, item in pairs (inventoryItemTable) do	
				local tempItem = item
				if tempItem and tempItem.itemInBag and tempItem.itemInBag:GetName() == tCurrItem:GetName() then
					table.insert(self.tItemSalv, tCurrItem)
					salvUpFlag = true
					break
				end
			end
			self:UpdateSalv()
			if self.wndSalvList:IsShown() and salvUpFlag == true then
				if tCurrItem ~= nil then
					local wndListItem = Apollo.LoadForm(self.xmlDoc, "ItemListSalvNode", self.wndSalvList, self)
					local bGivenQuest = tCurrItem:GetGivenQuest()
					local eItemQuality = tCurrItem and tCurrItem:GetItemQuality() or 1
					local cItemTooltipForm = Tooltip.GetItemTooltipForm(self, wndListItem:FindChild("LootSack"),tCurrItem, {bPrimary = true, bSelling = false, itemCompare = itemEquipped})
					wndListItem :FindChild("LootText"):SetTextColor(bGivenQuest and "white" or karEvalColors[eItemQuality])
					wndListItem :FindChild("LootSack"):SetSprite(bGivenQuest and "sprMM_QuestGiver" or tCurrItem:GetIcon())
					wndListItem :FindChild("LootSack"):SetTooltipForm(cItemTooltipForm)
					wndListItem :FindChild("RarityBracket"):SetSprite(bGivenQuest and "sprTooltip_Header_White" or karQualitySquareSprite[eItemQuality])
					if tCurrItem:GetSellPrice() ~= nil then
						wndListItem:FindChild("SwapList"):Show(true)
					else
						wndListItem:FindChild("SwapList"):Show(false)
					end
					
					wndListItem :FindChild("LootText"):SetText(tCurrItem:GetName())
					wndListItem:SetName(tCurrItem:GetName())
					--Print(wndListItem:GetName())
					self.wndSalvList:ArrangeChildrenVert()	
				end
			end
		else
			--Print("item not kept "..idx)
		end
	end
	--could be made into a seperate method
	for idxTwo = 4,1,-1 do	
		if tIndex[idxTwo] ~= 0 then
			self:RemoveItem(idxTwo)
			self:UpdateDisplay()
		end
	end
	--Print(self.nSalvIndex)
end

function SellSalvageKeep:OnKeepAll()
	--loop through each window
		--get item
		--if an item exists
			--remove item from window
			--update display
	local tIndex = {0,0,0,0}
	for idx = 1, 4 do		
		local tCurrEntryData = self.tEntryData[idx]
		local tCurrItem = tCurrEntryData and tCurrEntryData.itemInstance or false
	
		if tCurrItem then
			--Print("item kept "..idx)
			--self:RemoveItem(idx)
			tIndex[idx] = idx		
			--self:UpdateDisplay()
		else
			--Print("item not kept "..idx)
		end
	end
	--could be a seperate method
	for idxTwo = 4,1,-1 do	
		if tIndex[idxTwo] ~= 0 then
			self:RemoveItem(idxTwo)
			self:UpdateDisplay()
		end
	end
	self:NoItemClose()
end

function SellSalvageKeep:OnMiniSell(idx)
	local btnName = idx:GetName()
	local currSellEntry
	local currSellItem
	local currSellItemName
	local itemRemovalIdx
	if btnName == "MiniSell_1" then
		currSellEntry = self.tEntryData[1] 		
		itemRemovalIdx = 1		
	elseif btnName == "MiniSell_2" then
		currSellEntry = self.tEntryData[2] 
		itemRemovalIdx = 2		
	elseif btnName == "MiniSell_3" then
		currSellEntry = self.tEntryData[3] 
		itemRemovalIdx = 3	
	elseif btnName == "MiniSell_4" then
		currSellEntry = self.tEntryData[4] 
		itemRemovalIdx = 4	
	end
	currSellItem = currSellEntry and currSellEntry.itemInstance or false
	currSellItemName = currSellItem:GetName()
	local inventoryItemTable = GameLib.GetPlayerUnit():GetInventoryItems()
	for key, item in pairs (inventoryItemTable) do	
		local tempItem = item
		if tempItem and tempItem.itemInBag and tempItem.itemInBag:GetName() == currSellItemName then
			table.insert(self.tItemSell, currSellItem)
			break
		end
	end
	--remove item
	self:RemoveItem(itemRemovalIdx)
	self:UpdateDisplay()
	--Print(currSellItemName)
	self:NoItemClose()

end

function SellSalvageKeep:OnMiniSalv(idx)
	local btnName = idx:GetName()
	local currSalvEntry
	local currSalvItem
	local currSalvItemName
	local itemRemovalIdx
	if btnName == "MiniSalv_1" then
		currSalvEntry = self.tEntryData[1] 		
		itemRemovalIdx = 1		
	elseif btnName == "MiniSalv_2" then
		currSalvEntry = self.tEntryData[2] 
		itemRemovalIdx = 2		
	elseif btnName == "MiniSalv_3" then
		currSalvEntry = self.tEntryData[3] 
		itemRemovalIdx = 3	
	elseif btnName == "MiniSalv_4" then
		currSalvEntry = self.tEntryData[4] 
		itemRemovalIdx = 4	
	end

	self.nSalvIndex = 1
	currSalvItem = currSalvEntry and currSalvEntry.itemInstance or false
	currSalvItemName = currSalvItem:GetName()
	local salvUpFlag = false
	local inventoryItemTable = GameLib.GetPlayerUnit():GetInventoryItems()
	for key, item in pairs (inventoryItemTable) do	
		local tempItem = item
		if tempItem and tempItem.itemInBag and tempItem.itemInBag:GetName() == currSalvItemName then
			table.insert(self.tItemSalv, currSalvItem)
			salvUpFlag = true
			break
		end
	end
	--table.insert(self.tItemSalv, currSalvItem)
	--Print("mini salv pressed"..self.nSalvIndex)
	self:UpdateSalv()
		--remove item
	self:RemoveItem(itemRemovalIdx)
	self:UpdateDisplay()
	--if salv list window is being shown then add item to it
	local tCurrItem = currSalvItem
	if salvUpFlag ~= false and self.wndSalvList:IsShown() then
		if tCurrItem ~= nil then
				local wndListItem = Apollo.LoadForm(self.xmlDoc, "ItemListSalvNode", self.wndSalvList, self)
				local bGivenQuest = tCurrItem:GetGivenQuest()
				local eItemQuality = tCurrItem and tCurrItem:GetItemQuality() or 1
				local cItemTooltipForm = Tooltip.GetItemTooltipForm(self, wndListItem:FindChild("LootSack"),tCurrItem, {bPrimary = true, bSelling = false, itemCompare = itemEquipped})
				wndListItem :FindChild("LootText"):SetTextColor(bGivenQuest and "white" or karEvalColors[eItemQuality])
				wndListItem :FindChild("LootSack"):SetSprite(bGivenQuest and "sprMM_QuestGiver" or tCurrItem:GetIcon())
				wndListItem :FindChild("LootSack"):SetTooltipForm(cItemTooltipForm)
				wndListItem :FindChild("RarityBracket"):SetSprite(bGivenQuest and "sprTooltip_Header_White" or karQualitySquareSprite[eItemQuality])
					if tCurrItem:GetSellPrice() ~= nil then
						wndListItem:FindChild("SwapList"):Show(true)
					else
						wndListItem:FindChild("SwapList"):Show(false)
					end
					
				wndListItem :FindChild("LootText"):SetText(tCurrItem:GetName())
				wndListItem:SetName(tCurrItem:GetName())
				--Print(wndListItem:GetName())
				self.wndSalvList:ArrangeChildrenVert()	
			end
	end		
end

function SellSalvageKeep:OnMiniKeep(idx)
	local btnName = idx:GetName()
	local itemRemovalIdx
	
	if btnName == "MiniKeep_1" then		
		itemRemovalIdx = 1		
	elseif btnName == "MiniKeep_2" then
		itemRemovalIdx = 2		
	elseif btnName == "MiniKeep_3" then
		itemRemovalIdx = 3	
	elseif btnName == "MiniKeep_4" then
		itemRemovalIdx = 4	
	end
	
	self:RemoveItem(itemRemovalIdx)
	self:UpdateDisplay()
	self:NoItemClose()
end

function SellSalvageKeep:OnSalvagePress()
	--Print("salv pressed"..self.nSalvIndex)
    --table.remove(self.tItemSalv, self.nSalvIndex)
	if self.wndSalvList:IsShown() then
		self.wndSalvList:FindChild(self.SalvListItemRemover):Destroy()
		self.wndSalvList:ArrangeChildrenVert()
	end	
	--Print("Before"..self.nSalvIndex)
	self.nSalvIndex = self.nSalvIndex + 1
	self:UpdateSalv()
	self.tItemSalv[self.nSalvIndex - 1] = nil
	--Print("After"..self.nSalvIndex)		
end
function SellSalvageKeep:OnViewPress()
	if self.wndSelect:IsShown()== true then
		self.wndSelect:Show(false,true)
	else		
		local dropLeftOff, dropTopOff, dropRightOff, dropBotOff = self.wndLoot:GetAnchorOffsets()
		--Print("Left: "..dropLeftOff)
		--Print("Top: "..dropTopOff)
		--Print("Right: "..dropRightOff)
		--Print("Bot: "..dropBotOff)
		local newLeft = dropRightOff - 115
		local newTop = dropBotOff - 115
		--Print("Select Left: "..newLeft)
		--Print("Select Top: "..dropBotOff)
		--Print("Select Right: "..dropRightOff)
		--Print("Select Bot: "..newTop)
		self.wndSelect:SetAnchorOffsets(newLeft ,newTop + 100, dropRightOff, dropBotOff + 100)
		self.wndSelect:Invoke()
	end
end
function SellSalvageKeep:OnOptionPress()
	if self.wndSelect:IsShown() == true then
		self.wndSelect:Show(false, true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	end
	if self.wndOption:IsShown() == true then
		self.wndOption:Show(false, true)
		
	else	
		local dropLeftOff, dropTopOff, dropRightOff, dropBotOff = self.wndLoot:GetAnchorOffsets()

		local newRight = dropRightOff + 342
		local newTop = dropBotOff - 115

		self.wndOption:SetAnchorOffsets(dropRightOff ,dropTopOff, newRight, dropBotOff)
		
		self.wndOption:FindChild("Window"):Show(true)
		self.wndOption:Invoke()
	end	
end

function SellSalvageKeep:OnLootMove( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
	if self.wndSelect:IsShown()== true then
		self.wndSelect:Show(false,true)
	end
	if self.wndSellList:IsShown() == true then
		self.wndSellList:Show(false,true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndOption:IsShown() == true then
		self.wndOption:Show(false, true)
	end


end

----------------------------------------------------------------------------------------------
--salv managers
----------------------------------------------------------------------------------------------
function SellSalvageKeep:UpdateSalv()
	--Print("update salv"..self.nSalvIndex)
	local itemToBeSalvaged = self.tItemSalv[self.nSalvIndex]
	if itemToBeSalvaged ~= nil then
		self.wndLoot:FindChild("salvager"):SetActionData(GameLib.CodeEnumConfirmButtonType.SalvageItem, itemToBeSalvaged:GetInventoryId())
		self.SalvListItemRemover = itemToBeSalvaged:GetName()
	else
		self.wndLoot:FindChild("salvager"):SetActionData(nil)
	end		
end
----------------------------------------------------------------------------------------------
--good print
----------------------------------------------------------------------------------------------
function SellSalvageKeep:goodPrint(...)
    local args = { n = select("#", ...), ... }
    local toPrint = ""
    for i = 1, args.n do
        local arg
        if (args[i] ~= nil) then 
            arg = tostring(args[i])
        else
            arg = "nil"
        end
        toPrint = toPrint .. arg .. " "
    end
    Print(toPrint)
end

---------------------------------------------------------------------------------------------------
-- ViewSelect Functions
---------------------------------------------------------------------------------------------------

function SellSalvageKeep:OnSellListPress( wndHandler, wndControl, eMouseButton )
	if self.wndOption:IsShown() == true then
		self.wndOption:Show(false, true)
	end
	if self.wndSelect:IsShown() == true then
		self.wndSelect:Show(false, true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	else
		self.wndSellList:DestroyChildren()	
		local dropLeftOff, dropTopOff, dropRightOff, dropBotOff = self.wndLoot:GetAnchorOffsets()

		local newRight = dropRightOff + 342
		local newTop = dropBotOff - 115

		self.wndSellList:SetAnchorOffsets(dropRightOff ,dropTopOff, newRight, dropBotOff)
		self.wndSellList:Invoke()
	end
	if self.tItemSell[1] ~= nil then
		
		for idx = 1, #self.tItemSell do
			local wndListItem = Apollo.LoadForm(self.xmlDoc, "ItemListNode", self.wndSellList, self)
			local tCurrItem = self.tItemSell[idx]
			--Print(self.tItemSell[1])
			--local tCurrItem = tCurrSellEntry and tCurrSellEntry.itemInstance or false
			--Print(tCurrentSellEntry:GetName())
			if tCurrItem then
				local bGivenQuest = tCurrItem:GetGivenQuest()
				local eItemQuality = tCurrItem and tCurrItem:GetItemQuality() or 1
				local cItemTooltipForm = Tooltip.GetItemTooltipForm(self, wndListItem:FindChild("LootSack"),tCurrItem, {bPrimary = true, bSelling = false, itemCompare = itemEquipped})
				wndListItem :FindChild("LootText"):SetTextColor(bGivenQuest and "white" or karEvalColors[eItemQuality])
				wndListItem :FindChild("LootSack"):SetSprite(bGivenQuest and "sprMM_QuestGiver" or tCurrItem:GetIcon())
				wndListItem :FindChild("LootSack"):SetTooltipForm(cItemTooltipForm)
				wndListItem :FindChild("RarityBracket"):SetSprite(bGivenQuest and "sprTooltip_Header_White" or karQualitySquareSprite[eItemQuality])
				if tCurrItem:CanSalvage() then
					wndListItem:FindChild("SwapList"):Show(true)
				else
					wndListItem:FindChild("SwapList"):Show(false)
				end
			end
			--Print(tCurrItem.nCount)
			--if tCurrItem.nCount == 1 then
				wndListItem :FindChild("LootText"):SetText(tCurrItem:GetName())
			--else
				--wndListItem :FindChild("LootText"):SetText(String_GetWeaselString(Apollo.GetString("CombatLog_MultiItem"), tCurrItem.nCount, tCurrItem:GetName()))
			--end
				self.wndSellList:ArrangeChildrenVert()				
		end
	end
end

function SellSalvageKeep:OnCoinPursePress( wndHandler, wndControl, eMouseButton )
	--should be a method to reduce lines of code
	if self.wndOption:IsShown() == true then
		self.wndOption:Show(false, true)
	end
	if self.wndSelect:IsShown() == true then
		self.wndSelect:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	else
		local dropLeftOff, dropTopOff, dropRightOff, dropBotOff = self.wndLoot:GetAnchorOffsets()

		local newRight = dropRightOff + 247
		local newTop = dropTopOff + 230

		self.wndPurse:SetAnchorOffsets(dropRightOff ,newTop, newRight, dropBotOff)
		self.wndPurse:Invoke()
	end
end

function SellSalvageKeep:OnSalvListPress( wndHandler, wndControl, eMouseButton )
	--should be its own method
	if self.wndOption:IsShown() == true then
		self.wndOption:Show(false, true)
	end
	if self.wndSelect:IsShown() == true then
		self.wndSelect:Show(false, true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	else
		self.wndSalvList:DestroyChildren()	
		local dropLeftOff, dropTopOff, dropRightOff, dropBotOff = self.wndLoot:GetAnchorOffsets()

		local newRight = dropRightOff + 342
		local newTop = dropBotOff - 115

		self.wndSalvList:SetAnchorOffsets(dropRightOff ,dropTopOff, newRight, dropBotOff)
		self.wndSalvList:Invoke()
	end
	if #self.tItemSalv > 0 then
		--Print(#self.tItemSalv)
		
		for idx = 1, #self.tItemSalv do
			
			local tCurrItem = nil
			local wndListItem = nil
			if self.tItemSalv[idx] ~= nil then
				--Print(idx)
				tCurrItem = self.tItemSalv[idx] 
				wndListItem = Apollo.LoadForm(self.xmlDoc, "ItemListSalvNode", self.wndSalvList, self)
			end
			--local tCurrItem = tCurrSellEntry and tCurrSellEntry.itemInstance or false
			--Print(tCurrentSellEntry:GetName())
			if tCurrItem ~= nil and wndListItem ~= nil then
				local bGivenQuest = tCurrItem:GetGivenQuest()
				local eItemQuality = tCurrItem and tCurrItem:GetItemQuality() or 1
				local cItemTooltipForm = Tooltip.GetItemTooltipForm(self, wndListItem:FindChild("LootSack"),tCurrItem, {bPrimary = true, bSelling = false, itemCompare = itemEquipped})
				wndListItem :FindChild("LootText"):SetTextColor(bGivenQuest and "white" or karEvalColors[eItemQuality])
				wndListItem :FindChild("LootSack"):SetSprite(bGivenQuest and "sprMM_QuestGiver" or tCurrItem:GetIcon())
				wndListItem :FindChild("LootSack"):SetTooltipForm(cItemTooltipForm)
				wndListItem :FindChild("RarityBracket"):SetSprite(bGivenQuest and "sprTooltip_Header_White" or karQualitySquareSprite[eItemQuality])
					if tCurrItem:GetSellPrice() ~= nil then
						wndListItem:FindChild("SwapList"):Show(true)
					else
						wndListItem:FindChild("SwapList"):Show(false)
					end
					
				wndListItem :FindChild("LootText"):SetText(tCurrItem:GetName())
				wndListItem:SetName(tCurrItem:GetName())
				--Print(wndListItem:GetName())
				self.wndSalvList:ArrangeChildrenVert()	
			end
			
		end
	end	
end
function SellSalvageKeep:NoItemClose()
	if #self.tQueuedEntryData <= 0 and #self.tEntryData <= 0 then
		self.wndLoot:Show(false,true)
	end
	if self.wndSelect:IsShown() == true then
		self.wndSelect:Show(false, true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	end
	if self.wndOption:IsShown() == true then
		self.wndOption:Show(false, true)
	end
end

---------------------------------------------------------------------------------------------------
-- ItemListNode Functions
---------------------------------------------------------------------------------------------------

function SellSalvageKeep:OnRemoveSellItem( wndHandler, wndControl, eMouseButton )
	local listItem = wndHandler:GetParent():FindChild("LootText")
	local listItemName = listItem:GetText()
	for idx = 1, #self.tItemSell do
		local tCurrItem = self.tItemSell[idx]
		local currItemName = tCurrItem and tCurrItem:GetName()
		if currItemName == listItemName then
			table.remove(self.tItemSell, idx)
			wndHandler:GetParent():Destroy()
			break
		end
	end
	self.wndSellList:ArrangeChildrenVert()
end

function SellSalvageKeep:OnSellToSalv( wndHandler, wndControl, eMouseButton )
	local listItem = wndHandler:GetParent():FindChild("LootText")
	local listItemName = listItem:GetText()
	for idx = 1, #self.tItemSell do
		local tCurrItem = self.tItemSell[idx]
		local currItemName = tCurrItem and tCurrItem:GetName()
		if currItemName == listItemName and tCurrItem:CanSalvage() then
			table.remove(self.tItemSell, idx)
			wndHandler:GetParent():Destroy()
			table.insert(self.tItemSalv, tCurrItem)
			self.nSalvIndex = 1
			self:UpdateSalv()
			break
		end
	end
	self.wndSellList:ArrangeChildrenVert()

end

---------------------------------------------------------------------------------------------------
-- ItemListSalvNode Functions
---------------------------------------------------------------------------------------------------

function SellSalvageKeep:OnRemoveSalvItem( wndHandler, wndControl, eMouseButton )
	local listItem = wndHandler:GetParent():FindChild("LootText")
	local listItemName = listItem:GetText()
	for idx = 1, #self.tItemSalv do
		local tCurrItem = self.tItemSalv [idx]
		local currItemName = tCurrItem and tCurrItem:GetName()
		if currItemName == listItemName then
			table.remove(self.tItemSalv, idx)
			wndHandler:GetParent():Destroy()
			self:UpdateSalv()
			break
		end
	end
	self.wndSalvList:ArrangeChildrenVert()
end


function SellSalvageKeep:OnSalvToSell( wndHandler, wndControl, eMouseButton )
	local listItem = wndHandler:GetParent():FindChild("LootText")
	local listItemName = listItem:GetText()
	for idx = 1, #self.tItemSalv do
		local tCurrItem = self.tItemSalv [idx]
		local currItemName = tCurrItem and tCurrItem:GetName()
		if currItemName == listItemName and tCurrItem:GetSellPrice() ~= nil then
			table.remove(self.tItemSalv, idx)
			wndHandler:GetParent():Destroy()
			self:UpdateSalv()
			table.insert(self.tItemSell, tCurrItem)
			break
		end
	end
	self.wndSalvList:ArrangeChildrenVert()
end
---------------------------------------------------------------------------------------------------
--save and restore
---------------------------------------------------------------------------------------------------
function SellSalvageKeep:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		local itemTest = self.tItemSell[1]
		local saver = {
						nSaveVersion = knSaveVersion,
						itemSellIDs = {},
						itemSalvIDs = {},
						--options 
						--vac
						craftRadio = self.craftRadio,
						equipRadio = self.equipRadio,
						decorRadio = self.decorRadio,
						runeRadio = self.runeRadio,
						consumeRadio = self.consumeRadio,					
						ampRadio = self.ampRadio,
						bagRadio = self.bagRadio,
						costumeRadio = self.costumeRadio,
						pathRadio = self.pathRadio,
						miscRadio = self.miscRadio,
						schematicRadio = self.schematicRadio,
						warplotRadio = self.warplotRadio,
						-- coin purse
						shownRadio = self.shownRadio,
						noShowRadio = self.noShowRadio
				      }
		local nCount = 0
		for idx, item in pairs (self.tItemSell) do
			nCount = nCount + 1
			saver.itemSellIDs [nCount] = {}
			saver.itemSellIDs [nCount] = item:GetInventoryId()
		end
		nCount = 0
		for idx, item in pairs (self.tItemSalv) do
			nCount = nCount + 1
			saver.itemSalvIDs [nCount] = {}
			saver.itemSalvIDs [nCount] = item:GetInventoryId()
		end
		return saver
	end	
	return nil
end

function SellSalvageKeep:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character  then
		if not tSavedData or tSavedData.nSaveVersion ~= knSaveVersion then
			return
		end
		-- saved item lists
		if tSavedData.itemSellIDs then
			self.tItemSellIDs = tSavedData.itemSellIDs 
		end
		if tSavedData.itemSalvIDs then
			self.tItemSalvIDs = tSavedData.itemSalvIDs
		end
		
		--options menu
		-- vac
		if tSavedData.craftRadio ~= nil then
			self.craftRadio = tSavedData.craftRadio
		end
		if tSavedData.equipRadio ~= nil then
			self.equipRadio = tSavedData.equipRadio
		end
		if tSavedData.decorRadio ~= nil then
			self.decorRadio = tSavedData.decorRadio
		end
		if tSavedData.runeRadio ~= nil then
			self.runeRadio= tSavedData.runeRadio
		end
		if tSavedData.consumeRadio ~= nil then
			self.consumeRadio = tSavedData.consumeRadio
		end
		if tSavedData.ampRadio ~= nil then
			self.ampRadio = tSavedData.ampRadio
		end
		if tSavedData.bagRadio ~= nil then
			self.bagRadio = tSavedData.bagRadio
		end
		if tSavedData.costumeRadio ~= nil then
			self.costumeRadio = tSavedData.costumeRadio 
		end
		if tSavedData.pathRadio ~= nil then
			self.pathRadio = tSavedData.pathRadio 
		end
		if tSavedData.miscRadio ~= nil then
			self.miscRadio = tSavedData.miscRadio 
		end
		if tSavedData.schematicRadio ~= nil then
			self.schematicRadio = tSavedData.schematicRadio 
		end
		if tSavedData.warplotRadio ~= nil then
			self.warplotRadio = tSavedData.warplotRadio 
		end
		--coin purse
		if tSavedData.shownRadio then
			self.shownRadio = tSavedData.shownRadio
		end
		if tSavedData.noShowRadio then
			self.noShowRadio = tSavedData.noShowRadio
		end
	end
end

---------------------------------------------------------------------------------------------------
-- TutBase Functions
---------------------------------------------------------------------------------------------------

function SellSalvageKeep:OnTutExit( wndHandler, wndControl, eMouseButton )
	self.wndMain:Close()
	self.dongs=""
end

function SellSalvageKeep:OnTutLaunch( wndHandler, wndControl, eMouseButton )
	if self.wndSelect:IsShown() == true then
		self.wndSelect:Show(false, true)
	end
	if self.wndPurse:IsShown()== true and self.shownRadio == false then
		self.wndPurse:Show(false, true)
	end
	if self.wndSalvList:IsShown() == true then
		self.wndSalvList:Show(false, true)
	end
	if self.wndSellList:IsShown()== true then
		self.wndSellList:Show(false,true)
	end
	self.wndLoot:Invoke()
	
	self.wndMain:Close()
end

---------------------------------------------------------------------------------------------------
-- CoinPurse Functions
---------------------------------------------------------------------------------------------------

function SellSalvageKeep:OnExitPress( wndHandler, wndControl, eMouseButton )
	self.wndPurse:Close()
end
---------------------------------------------------------------------------------------------------
--restore list function
---------------------------------------------------------------------------------------------------
function SellSalvageKeep:RestoreDataLists()
	--display in interface menu
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Sell Salvage Keep", {"sskInterfaceMenuClicked", "", "spr_TargetFrame_ClassIcon_Fodder"})
		-- restore save lists
		local inventoryItemTable = GameLib.GetPlayerUnit():GetInventoryItems()
		for key, item in pairs (inventoryItemTable) do	
				local tempItem = item
				for idx, ID in pairs (self.tItemSellIDs) do	
					if tempItem and tempItem.itemInBag and tempItem.itemInBag:GetInventoryId() == ID then
						table.insert(self.tItemSell, tempItem.itemInBag)
					end
				end
				for idx, ID in pairs (self.tItemSalvIDs) do
					if tempItem and tempItem.itemInBag and tempItem.itemInBag:GetInventoryId() == ID and tempItem.itemInBag:CanSalvage() then
						table.insert(self.tItemSalv, tempItem.itemInBag)
						self.nSalvIndex = 1
						--Print(self.nSalvIndex)
						self:UpdateSalv()
					end
				end
		end
		-- restore options
		-- coin purse
		if self.shownRadio == true then
			--Print("coin shown es true")
			self.wndOption:FindChild("OptionsFill:coinBase:shownNo"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:coinBase:shownYes"):SetCheck(true)
			self.wndPurse:Show(true)
		end
		if self.noShowRadio == true then
			self.wndOption:FindChild("OptionsFill:coinBase:noShowNo"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:coinBase:noShowYes"):SetCheck(true)
		end
		-- vac
		--crafting mats
		if self.craftRadio == false then
			--Print("craft es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:craftYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:craftNo"):SetCheck(true)
		end
		--equipment
		if self.equipRadio == false then
			--Print("equip es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:equipYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:equipNo"):SetCheck(true)
		end
		--decor
		if self.decorRadio == false then
			--Print("decor es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:decorYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:decorNo"):SetCheck(true)
		end
		--runes
		if self.runeRadio == false then
			--Print("rune es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:RuneYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:RuneNo"):SetCheck(true)
		end
		--consumeables
		if self.consumeRadio == false then
			--Print("consume es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:ConsumeYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:ConsumeNo"):SetCheck(true)
		end
		--Amps
		if self.ampRadio == false then
			--Print("amp es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:AmpYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:AmpNo"):SetCheck(true)
		end
		--bags
		if self.bagRadio == false then
			--Print("bag es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:BagYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:BagNo"):SetCheck(true)
		end
		--costume
		if self.costumeRadio == false then
			--Print("consttume es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:CostumeYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:CostumeNo"):SetCheck(true)
		end
		--path
		if self.pathRadio == false then
			--Print("path es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:PathYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:PathNo"):SetCheck(true)
		end
		--Misc
		if self.miscRadio == false then
			--Print("misc es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:MiscYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:MiscNo"):SetCheck(true)
		end
		--schematic
		if self.schematicRadio == false then
			--Print("sche es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:SchematicYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:SchematicNo"):SetCheck(true)
		end
		--warplot
		if self.warplotRadio == false then
			--Print("war es false")
			self.wndOption:FindChild("OptionsFill:vacWnd1base:WarYes"):SetCheck(false)
			self.wndOption:FindChild("OptionsFill:vacWnd1base:WarNo"):SetCheck(true)
		end
							
end

---------------------------------------------------------------------------------------------------
-- Options Functions
---------------------------------------------------------------------------------------------------
-- coin purse always shown radio group
function SellSalvageKeep:OnCoinShownRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("shownCheck")
	if radioOption == 1 then
		self.shownRadio = true
		--Print("true")
	else
		self.shownRadio = false
		--Print("false")
	end
	
	if self.shownRadio == true and self.wndPurse:IsShown() ~= true then
		self.wndPurse:Show(true)
	else 
		self.wndPurse:Show(false)
	end
end
--coin purse never show radio group
function SellSalvageKeep:OnCoinNeverShowRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("NoShownCheck")
	if radioOption == 1 then
		self.noShowRadio = true
	else
		self.noShowRadio = false
	end
end
-- vac pop up on crafting material radio group
function SellSalvageKeep:OnVacCraftRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("craftMat")
	if radioOption == 1 then
		self.craftRadio = true
		--Print("true")
	else
		self.craftRadio = false
		--Print("false")
	end
end
-- vac pop up on equipment radio group
function SellSalvageKeep:OnVacEquipRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("equipCheck")
	if radioOption == 1 then
		self.equipRadio = true
		--Print("true")
	else
		self.equipRadio = false
		--Print("false")
	end
end
-- vac pop up on decor radio group
function SellSalvageKeep:OnVacDecorRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("decorCheck")
	if radioOption == 1 then
		self.decorRadio = true
		--Print("true")
	else
		self.decorRadio = false
		--Print("false")
	end
end
-- vac pop up on runes radio group
function SellSalvageKeep:OnVacRuneRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("runeCheck")
	if radioOption == 1 then
		self.runeRadio = true
		--Print("true")
	else
		self.runeRadio = false
		--Print("false")
	end
end
-- vac pop up on consumeables radio group
function SellSalvageKeep:OnVacConsumeRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("consumeCheck")
	if radioOption == 1 then
		self.consumeRadio = true
		--Print("true")
	else
		self.consumeRadio = false
		--Print("false")
	end
end

function SellSalvageKeep:OnVacAmpRadioCheck( wndHandler, wndControl, eMouseButton )
	local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("AmpCheck")
	if radioOption == 1 then
		self.ampRadio = true
		--Print("true")
	else
		self.ampRadio = false
		--Print("false")
	end

end

function SellSalvageKeep:OnVacBagRadioCheck( wndHandler, wndControl, eMouseButton )
local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("BagCheck")
	if radioOption == 1 then
		self.bagRadio = true
		--Print("true")
	else
		self.bagRadio = false
		--Print("false")
	end

end

function SellSalvageKeep:OnVacCostumeRadioCheck( wndHandler, wndControl, eMouseButton )
local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("CostumeCheck")
	if radioOption == 1 then
		self.costumeRadio = true
		--Print("true")
	else
		self.costumeRadio = false
		--Print("false")
	end

end

function SellSalvageKeep:OnVacPathRadioCheck( wndHandler, wndControl, eMouseButton )
local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("PathCheck")
	if radioOption == 1 then
		self.pathRadio = true
		--Print("true")
	else
		self.pathRadio = false
		--Print("false")
	end

end

function SellSalvageKeep:OnVacMiscRadioCheck( wndHandler, wndControl, eMouseButton )
local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("MiscCheck")
	if radioOption == 1 then
		self.miscRadio = true
		--Print("true")
	else
		self.miscRadio = false
		--Print("false")
	end

end

function SellSalvageKeep:OnVacSchematicRadioCheck( wndHandler, wndControl, eMouseButton )
local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("schematicCheck")
	if radioOption == 1 then
		self.schematicRadio = true
		--Print("true")
	else
		self.schematicRadio = false
		--Print("false")
	end

end

function SellSalvageKeep:OnVacWarplotRadioCheck( wndHandler, wndControl, eMouseButton )
local wndParent = wndControl:GetParent()
	local radioOption = wndParent:GetRadioSel("warCheck")
	if radioOption == 1 then
		self.warplotRadio = true
		--Print("true")
	else
		self.warplotRadio = false
		--Print("false")
	end

end

-----------------------------------------------------------------------------------------
-- SellSalvageKeep Instance
-----------------------------------------------------------------------------------------------
local SellSalvageKeepInst = SellSalvageKeep:new()
SellSalvageKeepInst:Init()
